import logging
import os
import zipfile

from nak.config import Config
from nak.gateway import Gateway
from nak.settings import (CONFIG_FILE, ENV_FILE, LOG_COLOR,
                          ZIP_DESTINATION_DIRECTORY, ZIP_DESTINATION_PATH)
from nak.utils import (get_all_file, get_error_from_response, hide_variable,
                       progress_bar)

logging.basicConfig(
    format='%(asctime)s %(levelname)s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S',
)


class Command(object):
    def __init__(self):
        self.config = Config()
        self.gateway = Gateway(
            email=self.config.email,
            password=self.config.password,
            client_id=self.config.client_id
        )

    def setup(self, parser=None):
        self.config.client_id = input(
            f"App Client ID [{hide_variable(self.config.client_id)}]: ") or self.config.client_id
        self.config.email = input(
            f"App Email [{self.config.email}]: ") or self.config.email
        self.config.password = input(
            f"App Password [{hide_variable(self.config.password, True)}]: ") or self.config.password
        self.config.save()

        logging.info(LOG_COLOR.SUCCESS.format(
            message=f'App with client_id {self.config.client_id} has been setup successfully.'
        ))

    def build(self, parser=None):
        if not (os.path.exists(ENV_FILE) or os.path.exists(CONFIG_FILE)):
            raise TypeError(LOG_COLOR.ERROR.format(
                message=(
                    'Unable to locate config or env file. '
                    'You can configure config or env file by running "nak setup".')))

        app_name = os.getcwd().split('/')[-1]
        current_path = "."

        file_list = get_all_file(path=current_path)
        for file in file_list:
            logging.info(LOG_COLOR.INFO.format(message=f'file: {file}'))

        destination_file = ZIP_DESTINATION_PATH.format(app_name=app_name)

        # create directories
        if not os.path.exists(ZIP_DESTINATION_DIRECTORY):
            os.mkdir(ZIP_DESTINATION_DIRECTORY)

        new_zip = zipfile.ZipFile(destination_file, 'w')
        for file in progress_bar(file_list, prefix='Progress:', suffix='Complete', length=50):
            new_zip.write(file)

        new_zip.close()
        logging.info(LOG_COLOR.SUCCESS.format(message='Build successfully.'))

    def push(self, parser=None):
        if not os.path.exists(ZIP_DESTINATION_DIRECTORY):
            raise TypeError(LOG_COLOR.ERROR.format(
                message=(
                    'Unable to locate config or env file. '
                    'You can configure config or env file by running "nak setup".')))

        zip_file_list = get_all_file(path=f"./{ZIP_DESTINATION_DIRECTORY}", required_extension='.zip')

        if not zip_file_list:
            raise TypeError(LOG_COLOR.ERROR.format(message='Please run build before push command.'))

        latest_build_file = sorted(zip_file_list, key=lambda file_name: file_name, reverse=True)[0]
        file_name = latest_build_file.split("/")[-1]

        logging.info(LOG_COLOR.INFO.format(message=f'Pushing to app with client_id {self.config.client_id}'))
        logging.info(LOG_COLOR.INFO.format(message=f'with filename {file_name}'))
        logging.info(LOG_COLOR.INFO.format(message=f'by username {self.config.email}'))

        files = {'file': (latest_build_file, open(f'{latest_build_file}', 'rb'))}

        response = self.gateway.update_app(files=files)
        if not response.ok:
            error_msg = get_error_from_response(response)
            logging.info(LOG_COLOR.ERROR.format(message=f'Upload file to server failed. {error_msg}'))
            return

        logging.info(LOG_COLOR.SUCCESS.format(message='Push update file to app successfully.'))



#!/usr/bin/env python
import logging

from requests.exceptions import HTTPError

from nak.parser import Parser

logging.basicConfig(
    format='%(asctime)s %(levelname)s %(message)s',
    level=logging.INFO,
    datefmt='%Y-%m-%d %H:%M:%S'
)


def main():
    parser = Parser().create_parser()
    args = parser.parse_args()
    try:
        args.func(args)
    except AttributeError:
        print('Use nak -h or --help to see available commands')
    except (TypeError, HTTPError) as e:
        # print new line for support error on process progress bar
        print()
        logging.exception(e, exc_info=False)
    except KeyboardInterrupt:
        pass


if __name__ == '__main__':
    main()



from unittest import TestCase
from unittest.mock import MagicMock, patch

from nak import utils


class TestUtils(TestCase):
    ####
    # hide_variable
    ####
    def test_hide_variable_should_return_string_we_expect(self):
        actual = utils.hide_variable('12212112345666')
        expected = '********12345666'
        assert actual == expected

    def test_hide_variable_with_all_true_should_return_string_we_expect(self):
        actual = utils.hide_variable('12212112345666', all=True)
        expected = '********'
        assert actual == expected

    ####
    # get_error_from_response
    ####
    def test_get_error_from_response_should_return_correctly_error_msg(self):
        response = MagicMock()
        response.json.return_value = dict(
            file=['required', ]
        )

        actual = utils.get_error_from_response(response)
        assert actual == '"file" : required'

        response.json.return_value = dict(
            error='file is required'
        )

        actual = utils.get_error_from_response(response)
        assert actual == 'file is required'

    ####
    # get_all_file
    ####
    @patch('os.listdir')
    def test_get_all_file_with_allow_file_should_return_correctly_file_list(self, mock_listdir):
        mock_listdir.return_value = [
            'test1.html', 'test2.html', 'test.json'
        ]

        actual = utils.get_all_file('.')

        assert actual == ['./test1.html', './test2.html', './test.json']

    @patch('os.listdir')
    def test_get_all_file_with_not_allow_file_should_return_correctly_file_list(self, mock_listdir):
        mock_listdir.return_value = [
            '.env', 'config.yml', '.tmp', 'test1.a', 'test2.b', 'test.c'
        ]

        actual = utils.get_all_file('.')
        assert actual == []

    @patch('os.listdir')
    def test_get_all_file_with_required_extension_file_should_return_correctly_file_list(self, mock_listdir):
        mock_listdir.return_value = [
            'test1.a', 'test2.b', 'test.c', 'test_zip1.zip', 'test_zip2.zip'
        ]

        actual = utils.get_all_file('.', required_extension='.zip')
        assert actual == ['./test_zip1.zip', './test_zip2.zip']



import logging
import os

import yaml
from decouple import config as env_config

from nak.settings import CONFIG_FILE, ENV_FILE, LOG_COLOR


class Config(object):
    email = None
    password = None
    client_id = None

    def __init__(self):
        configs, env = self.read_config()

        self.client_id = configs.get('client_id')
        self.email = env.get('email')
        self.password = env.get('password')

    def read_config(self):
        configs = {}
        env = {}

        if os.path.exists(CONFIG_FILE):
            with open(CONFIG_FILE, "r") as yamlfile:
                configs = yaml.load(yamlfile, Loader=yaml.FullLoader) or {}
                yamlfile.close()

        if os.path.exists(ENV_FILE):
            env_config.search_path = os.getcwd()
            env['email'] = env_config('email', None)
            env['password'] = env_config('password', None)

        return configs, env

    def validate_config(self):
        error_msgs = []
        if not self.client_id:
            error_msgs.append('client_id')
        if not self.email:
            error_msgs.append('email')
        if not self.password:
            error_msgs.append('password')

        if error_msgs:
            message = ', '.join(error_msgs)
            pluralize = 'is' if len(error_msgs) == 1 else 'are'
            raise TypeError(LOG_COLOR.ERROR.format(
                message=f'argument {message} {pluralize} required.'
            ))

        return True

    def write_config(self):
        configs, env = self.read_config()
        new_configs = {
            'client_id': self.client_id
        }
        if not configs or configs != new_configs:
            with open(CONFIG_FILE, 'w') as yamlfile:
                yaml.dump({
                    'client_id': self.client_id
                }, yamlfile)
                yamlfile.close()

            logging.info(LOG_COLOR.INFO.format(message='Configuration was updated.'))

        new_env = {
            'email': self.email,
            'password': self.password
        }
        if not env or env != new_env:
            with open(ENV_FILE, 'w') as envfile:
                environments = "\n".join([
                    f'email={self.email}',
                    f'password={self.password}'
                ])
                envfile.writelines(environments)
            logging.info(LOG_COLOR.INFO.format(message='Environment was updated.'))

    def save(self):
        if not self.validate_config():
            return
        self.write_config()



import requests

from nak.settings import API_URL


class Gateway:
    def __init__(self, email, password, client_id):
        self.client_id = client_id
        self.email = email
        self.password = password

    def _request(self, request_type, url, payload={}, files=None):
        authenticate = requests.auth.HTTPBasicAuth(self.email, self.password)
        return requests.request(request_type, url, data=payload, files=files, auth=authenticate)

    def update_app(self, files):
        url = f"{API_URL}/api/apps/{self.client_id}/"
        return self._request("PATCH", url, files=files)



from unittest import TestCase
from unittest.mock import MagicMock, call, patch

from requests.auth import HTTPBasicAuth

from nak.gateway import Gateway
from nak.settings import API_URL


class TestGateway(TestCase):
    def setUp(self):
        self.client_id = 'ABCD1234'
        self.email = 'test@test.com'
        self.password = 'password'

        self.gateway = Gateway(self.client_id, self.password, self.client_id)
        self.mock_zip_file = MagicMock()

    ####
    # _request
    ####
    @patch('nak.gateway.requests', autospec=True)
    def test_request_should_call_requests_correctly(self, mock_requests):
        mock_requests.auth.HTTPBasicAuth.return_value = authenticate = HTTPBasicAuth(
            self.email, self.password)

        request_type = 'POST'
        url = 'test.com'
        payload = {}
        files = {'file': ('tmp/test.zip', self.mock_zip_file)}

        self.gateway._request(request_type, url, payload=payload, files=files)

        expected_calls = [call(
            'POST', 'test.com',
            data={},
            files=files,
            auth=authenticate)
        ]
        assert mock_requests.request.mock_calls == expected_calls

    @patch('nak.gateway.requests')
    def test_gateway_update_app_success_should_call_request_correctly(self, mock_requests):
        mock_requests.request.return_value.json.return_value = {"a": "b"}
        mock_requests.request.return_value.ok = True
        mock_requests.request.return_value.headers = {'content-type': 'application/json'}
        mock_requests.auth.HTTPBasicAuth.return_value = authenticate = HTTPBasicAuth(
            self.email, self.password)

        self.gateway.update_app(self.mock_zip_file)

        assert mock_requests.request.mock_calls == [
            # call request
            call(
                'PATCH', f'{API_URL}/api/apps/ABCD1234/',
                data={}, files=self.mock_zip_file, auth=authenticate),
        ]



import os
import time

ENV_FILE_NAME = './.env'
ENV_FILE = os.path.abspath(ENV_FILE_NAME)

CONFIG_FILE_NAME = './config.yml'
CONFIG_FILE = os.path.abspath(CONFIG_FILE_NAME)

API_URL = 'https://accounts.29next.com'

ZIP_FILE_FORMAT = "{app_name}-" + time.strftime("%Y%m%d%H%M%S")
ZIP_DESTINATION_DIRECTORY = '.tmp'
ZIP_DESTINATION_PATH = f'./{ZIP_DESTINATION_DIRECTORY}/{ZIP_FILE_FORMAT}.zip'
ZIP_EXCLUDE_FILES = ['.env', 'config.yml', ZIP_DESTINATION_DIRECTORY]

ALLOW_FILE_EXTENSIONS = [
    # CONTENT_FILE_EXTENSIONS
    '.html', '.json', '.css', '.scss', '.js',

    # MEDIA_FILE_EXTENSIONS
    '.woff2', '.gif', '.ico', '.png', '.jpg', '.jpeg', '.svg', '.eot', '.tff', '.ttf', '.woff',
    '.webp', '.mp4', '.webm', '.mp3', '.pdf'
]


class LOG_COLOR:
    ERROR = '\x1b[31;10m{message}\x1b[0m'
    SUCCESS = '\x1b[32;10m{message}\x1b[0m'
    INFO = '\x1b[36;10m{message}\x1b[0m'



import unittest
from unittest.mock import MagicMock, call, mock_open, patch

from nak.command import Command
from nak.settings import LOG_COLOR


class TestCommand(unittest.TestCase):
    @patch("yaml.load", autospec=True)
    @patch('nak.command.Gateway', autospec=True)
    def setUp(self, mock_gateway, mock_load_yaml):
        mock_load_yaml.return_value = {}

        with patch('builtins.open', mock_open(read_data='yaml data')):
            self.command = Command()

            self.mock_file = mock_open(read_data='test data')
            self.mock_gateway = mock_gateway

    #####
    # setup
    #####
    @patch('builtins.input', autospec=True)
    def test_setup_command_with_not_fill_any_input_should_raise_error_with_argument_required(self, mock_input):
        mock_input.side_effect = [None, None, None]

        with self.assertRaises(TypeError) as error:
            self.command.setup()

        assert str(error.exception) == LOG_COLOR.ERROR.format(
            message='argument client_id, email, password are required.')

    @patch("nak.command.Config.write_config", autospec=True)
    @patch('builtins.input', autospec=True)
    def test_setup_command_with_not_fill_any_input_but_already_have_config_should_not_raise(
        self, mock_input, mock_write_config
    ):
        mock_input.side_effect = [None, None, None]
        self.command.config.client_id = '123456'
        self.command.config.email = 'test@29next.com'
        self.command.config.password = 'password'

        self.command.setup()
        mock_write_config.assert_called_once()

    @patch("nak.command.Config.write_config", autospec=True)
    @patch('builtins.input', autospec=True)
    def test_setup_fill_correct_value_should_call_create_config_and_env_file_correctly(
        self, mock_input, mock_write_config
    ):
        mock_input.side_effect = [
            123456,  # client id
            'test@29next.com',  # email
            'password'  # password
        ]
        with self.assertLogs(level='INFO') as log:
            self.command.setup()

        assert log.output == ['INFO:root:' + LOG_COLOR.SUCCESS.format(
            message='App with client_id 123456 has been setup successfully.')]

        assert mock_write_config.call_count == 1

    #####
    # build
    #####
    @patch("os.path.exists", autospec=True)
    @patch("nak.command.zipfile", autospec=True)
    @patch("nak.command.get_all_file", autospec=True)
    @patch("nak.command.Config.write_config", autospec=True)
    def test_build_with_wrong_directory_should_raise_error_correctly(
        self, mock_write_config, mock_get_file, mock_zipfile, mock_path_exists
    ):
        mock_get_file.return_value = ["test1.file", "test2.file"]
        mock_path_exists.side_effect = [False, False]

        with self.assertRaises(TypeError) as error:
            self.command.build()

        assert str(error.exception) == LOG_COLOR.ERROR.format(
            message=(
                'Unable to locate config or env file. '
                'You can configure config or env file by running "nak setup".'))
        assert mock_zipfile.mock_calls == []

    @patch("os.path.exists")
    @patch("nak.command.zipfile", autospec=True)
    @patch("nak.command.get_all_file", autospec=True)
    @patch("nak.command.Config.write_config", autospec=True)
    def test_build_with_correct_directory_should_create_zip_file_correctly(
        self, mock_write_config, mock_get_file, mock_zipfile, mock_path_exists
    ):

        mock_get_file.return_value = ["test1.file", "test2.file"]
        mock_path_exists.side_effect = [
            # check config dir at build command
            True,
            # check env file at build command
            True
        ]
        with self.assertLogs(level='INFO') as log:
            self.command.build()

        assert mock_zipfile.ZipFile.return_value.mock_calls == [
            call.write('test1.file'),
            call.write('test2.file'),
            call.close(),
        ]
        assert log.output == [
            f"INFO:root:{LOG_COLOR.INFO.format(message='file: test1.file')}",
            f"INFO:root:{LOG_COLOR.INFO.format(message='file: test2.file')}",
            f"INFO:root:{LOG_COLOR.SUCCESS.format(message='Build successfully.')}"
        ]

    #####
    # push
    #####
    @patch("os.path.exists", autospec=True)
    @patch("nak.command.get_all_file", autospec=True)
    @patch("nak.command.Config.write_config", autospec=True)
    def test_push_with_wrong_directory_should_raise_error_correctly(
        self, mock_write_config, mock_get_file, mock_path_exists
    ):
        mock_get_file.return_value = ["test1.file", "test2.file"]
        mock_path_exists.side_effect = [
            # check self dir at push command
            False,
            # check self dir at push command
            False,
        ]

        with self.assertRaises(TypeError) as error:
            self.command.push()

        assert str(error.exception) == LOG_COLOR.ERROR.format(
            message=(
                'Unable to locate config or env file. '
                'You can configure config or env file by running "nak setup".'))

    @patch("builtins.open", autospec=True)
    @patch("os.path.exists", autospec=True)
    @patch("nak.command.get_all_file", autospec=True)
    @patch("nak.command.Config.write_config", autospec=True)
    def test_push_with_failed_on_server_should_log_error_correctly(
        self, mock_write_config, mock_get_file, mock_path_exists, mock_open_file
    ):
        mock_get_file.return_value = ["test1-20200101010101.zip", "test2-20200101010102.zip"]
        mock_path_exists.side_effect = [
            # check envfile at read_config
            True,
            # check self dir at push command
            True
        ]
        self.command.config.client_id = '123456'
        self.command.config.email = 'test@29next.com'

        mock_open_file.return_value = file = MagicMock()
        mock_response = self.mock_gateway.return_value.update_app.return_value
        mock_response.ok = False
        mock_response.json.return_value = {"error": "file size limit"}

        with self.assertLogs(level='INFO') as log:
            self.command.push()

        self.mock_gateway.return_value.update_app.assert_called_once_with(
            files={
                # send with latest file
                'file': ('test2-20200101010102.zip', file)
            }
        )
        assert log.output == [
            f'INFO:root:{LOG_COLOR.INFO.format(message="Pushing to app with client_id 123456")}',
            f'INFO:root:{LOG_COLOR.INFO.format(message="with filename test2-20200101010102.zip")}',
            f'INFO:root:{LOG_COLOR.INFO.format(message="by username test@29next.com")}',
            f"INFO:root:{LOG_COLOR.ERROR.format(message='Upload file to server failed. file size limit')}"
        ]

    @patch("builtins.open", autospec=True)
    @patch("os.path.exists", autospec=True)
    @patch("nak.command.get_all_file", autospec=True)
    @patch("nak.command.Config.write_config", autospec=True)
    def test_push_with_correct_directory_should_call_gateway_correctly(
        self, mock_write_config, mock_get_file, mock_path_exists, mock_open_file
    ):
        mock_get_file.return_value = ["test1-20200101010101.zip", "test2-20200101010102.zip"]
        mock_path_exists.side_effect = [
            # check envfile at read_config
            True,
            # check self dir at push command
            True
        ]
        self.command.config.client_id = '123456'
        self.command.config.email = 'test@29next.com'

        mock_open_file.return_value = file = MagicMock()
        mock_response = self.mock_gateway.return_value.update_app.return_value
        mock_response.ok = True

        with self.assertLogs(level='INFO') as log:
            self.command.push()

        self.mock_gateway.return_value.update_app.assert_called_once_with(
            files={
                # send with latest file
                'file': ('test2-20200101010102.zip', file)
            }
        )
        assert log.output == [
            f'INFO:root:{LOG_COLOR.INFO.format(message="Pushing to app with client_id 123456")}',
            f'INFO:root:{LOG_COLOR.INFO.format(message="with filename test2-20200101010102.zip")}',
            f'INFO:root:{LOG_COLOR.INFO.format(message="by username test@29next.com")}',
            f"INFO:root:{LOG_COLOR.SUCCESS.format(message='Push update file to app successfully.')}"
        ]



from setuptools import find_packages, setup

__version__ = '1.0.1'

tests_require = [
    "flake8==3.9.2",
    "nose==1.3.7"
]

with open('README.md', 'r') as fh:
    long_description = fh.read()


setup(
    name='next-app-kit',
    author="29next",
    author_email="dev@29next.com",
    url='https://github.com/29next/app-kit',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=__version__,
    install_requires=[
        "PyYAML>=5.4",
        "requests>=2.25",
        "python-decouple>=3.6"
    ],
    entry_points={
        'console_scripts': [
            'nak = nak.nak:main',
        ],
    },
    packages=find_packages(),
    python_requires='>=3.6'
)



from unittest import TestCase
from unittest.mock import call, mock_open, patch

from nak.config import Config
from nak.settings import LOG_COLOR


class TestConfig(TestCase):
    def setUp(self):
        self.config = Config()
        self.config.email = 'test@29next.com'
        self.config.password = 'password'
        self.config.client_id = '123456'

    ####
    # read_config
    ####
    @patch('builtins.open', mock_open(read_data='yaml data'))
    @patch("yaml.load", autospec=True)
    @patch("nak.config.env_config", autospec=True)
    @patch("os.path.exists", autospec=True)
    def test_read_config_should_read_config_from_file_and_return_config_correctly(
        self, mock_patch_exists, mock_env, mock_load_yaml
    ):
        mock_patch_exists.return_value = True
        mock_load_yaml.return_value = expected_config = {
            'client_id': 123456
        }
        mock_env.side_effect = ['test@29next.com', 'password']

        configs, env = self.config.read_config()
        expected_env = {
            'email': 'test@29next.com',
            'password': 'password'
        }
        assert configs == expected_config
        assert env == expected_env

    ####
    # validate_config
    ####
    def test_validate_config_should_raise_error_we_expect(self):
        with self.assertRaises(TypeError) as error:
            self.config.client_id = None
            self.config.email = 'test@test.com'
            self.config.password = 1234
            self.config.validate_config()
        assert str(error.exception) == LOG_COLOR.ERROR.format(message='argument client_id is required.')

        with self.assertRaises(TypeError) as error:
            self.config.client_id = 123456
            self.config.email = None
            self.config.password = 1234
            self.config.validate_config()
        assert str(error.exception) == LOG_COLOR.ERROR.format(message='argument email is required.')

        with self.assertRaises(TypeError) as error:
            self.config.client_id = 123456
            self.config.email = 'test@test.com'
            self.config.password = None
            self.config.validate_config()
        assert str(error.exception) == LOG_COLOR.ERROR.format(message='argument password is required.')

        with self.assertRaises(TypeError) as error:
            self.config.client_id = None
            self.config.email = None
            self.config.password = None
            self.config.validate_config()
        assert str(error.exception) == LOG_COLOR.ERROR.format(
            message='argument client_id, email, password are required.')

    def test_validate_with_config_not_none_should_not_raise_error(self):
        assert self.config.validate_config()

    # ####
    # # write_config
    # ####
    @patch("yaml.dump", autospec=True)
    @patch("nak.config.Config.read_config", autospec=True)
    @patch("os.path.exists", autospec=True)
    def test_write_config_with_new_config_should_call_update_file_config_correctly(
        self, mock_patch_exists, mock_read_config, mock_dump_yaml
    ):
        mock_patch_exists.return_value = True
        mock_dump_yaml.return_value = 'yaml data'
        mock_read_config.return_value = {
            'client_id': 123456,
        }, {
            'email': 'test@29next.com',
            'password': 'password',
        }

        # new config
        self.config.client_id = 56789
        self.config.user_name = 'test2@test.com'
        self.config.password = "password2"

        # self.config.write_config()
        with patch('builtins.open', mock_open()):
            with open('config.yml') as f:
                self.config.write_config()
                mock_dump_yaml.assert_called_once_with({
                    'client_id': 56789,
                }, f)

            with open('.env') as env:
                assert env.writelines.mock_calls == [call('email=test@29next.com\npassword=password2')]

    ####
    # save
    ####
    @patch("nak.config.Config.write_config", auto_space=True)
    def test_save_config_with_validate_true_true_should_call_write_config(self, mock_write_config):
        self.config.save()
        mock_write_config.assert_called_once_with()

    @patch("nak.config.Config.write_config", auto_space=True)
    def test_save_config_with_validate_false_false_should_call_write_config(self, mock_write_config):
        self.config.client_id = None

        with self.assertRaises(TypeError) as error:
            self.config.save()

        assert str(error.exception) == LOG_COLOR.ERROR.format(message='argument client_id is required.')
        mock_write_config.assert_not_called()



import os
import time
from pathlib import Path

from nak.settings import ALLOW_FILE_EXTENSIONS, ZIP_EXCLUDE_FILES


def get_latest_zip(pathfile):
    return Path(os.path.relpath(pathfile)).as_posix()


def progress_bar(iterable, prefix='', suffix='', decimals=1, length=100, fill='█', printEnd="\r"):
    """
    Call in a loop to create terminal progress bar
    @params:
        iteration   - Required  : current iteration (Int)
        total       - Required  : total iterations (Int)
        prefix      - Optional  : prefix string (Str)
        suffix      - Optional  : suffix string (Str)
        decimals    - Optional  : positive number of decimals in percent complete (Int)
        length      - Optional  : character length of bar (Int)
        fill        - Optional  : bar fill character (Str)
        printEnd    - Optional  : end character (e.g. "\r", "\r\n") (Str)
    """
    total = len(iterable)

    if total == 0:
        return

    # Progress Bar Printing Function
    def print_progress_bar(iteration):
        percent = ("{0:." + str(decimals) + "f}").format(100 * (iteration / float(total)))
        filledLength = int(length * iteration // total)
        bar = fill * filledLength + '-' * (length - filledLength)
        current_time = time.strftime('%Y-%m-%d %H:%M:%S')
        print(f'\r{current_time} INFO {prefix} | \033[1;32m{bar}\033[1;0m| {percent}% {suffix}', end=printEnd)

    # Initial Call
    print_progress_bar(0)
    # Update Progress Bar
    for i, item in enumerate(iterable):
        yield item
        print_progress_bar(i + 1)
    # Print New Line on Complete
    print()


def get_all_file(path, required_extension=None):
    list_file = os.listdir(path)
    all_files = []
    for entry in list_file:
        extension = os.path.splitext(entry)[1]
        if any([exclude_file in entry for exclude_file in ZIP_EXCLUDE_FILES]) or \
                (extension != required_extension and not (os.path.isdir(entry) or extension in ALLOW_FILE_EXTENSIONS)):
            continue

        fullPath = os.path.join(path, entry)
        if os.path.isdir(fullPath):
            all_files = all_files + get_all_file(fullPath)
        else:
            all_files.append(fullPath)
    return all_files


def hide_variable(value, all=False):
    if not value:
        return
    return "********" if all else f"********{value[-8:]}"


def get_error_from_response(response):
    result = response.json()
    error_msg = ""
    for key, value in result.items():
        if type(value) == list:
            error_msg += f'"{key}" : {" ".join(value)}'
        else:
            error_msg += value

    return error_msg



import argparse

from .command import Command


class Parser:
    def __init__(self):
        self.command = Command()
        pass

    def create_parser(self):
        # create the top-level parser
        parser = argparse.ArgumentParser(
            description='''
Usage:
    nak [command]

available commands:
    setup         Initialize a new app, will create config at config.yml and environment at .env file
    build         Compress file as zip
    push          Upload file to app server
''',
            usage=argparse.SUPPRESS,
            epilog='Use "nak [command] --help" for more information about a command.',
            formatter_class=argparse.RawTextHelpFormatter,
            add_help=argparse.SUPPRESS
        )
        subparsers = parser.add_subparsers(title='Available Commands', help=argparse.SUPPRESS)

        # create the parser for the "setup" command
        parser_setup = subparsers.add_parser(
            'setup',
            help='setup',
            usage=argparse.SUPPRESS,
            description='''
Usage:
    nak setup
        ''',
            formatter_class=argparse.RawTextHelpFormatter
        )
        parser_setup.set_defaults(func=self.command.setup)
        # create the parser for the "build" command
        parser_build = subparsers.add_parser(
            'build',
            help='build',
            usage=argparse.SUPPRESS,
            description='''
Usage:
    nak build
        ''',
            formatter_class=argparse.RawTextHelpFormatter
        )
        parser_build.set_defaults(func=self.command.build)
        # create the parser for the "push" command
        parser_push = subparsers.add_parser(
            'push',
            help='push',
            usage=argparse.SUPPRESS,
            description='''
Usage:
    nak push
        ''',
            formatter_class=argparse.RawTextHelpFormatter
        )
        parser_push.set_defaults(func=self.command.push)
        return parser
